""" Frosbite product interfaces

$Id: interfaces.py,v 1.1.1.1 2005/09/19 12:25:21 tseaver Exp $
"""
from zope.interface import Interface
from zope.interface import Attribute

class ISkinFreezer(Interface):
    """ Generate a Zope2 product for creating "frozen" skin objects.

    o These objects are filesystem (hence, trusted) analogues for skin
      folders:
      
      - Their methods are synthesized from skin scripts, and run as
        trusted code.

      - Their templates are synthesized from PageTemplates in skins, and
        may be run as trusted code.
    """
    def listCandidates(skins_tool=None, skin_name=None):
        """ Return a list of mappings describing candidates for freezing.

        o 'skins_tool' is the tool to search;  if None, acquire it as
          'portal_skins'.  If a string, acquire it.

        o 'skin_name' is the name of the skin whose path we are to search.
          If None, use the tool's default skin.

        o Returned mappings include the following keys:

          'path' -- the path to the skin method, relative to the tool.

          'product' -- the product which supplies the method (None for
            TTW methods)

          'meta_type' -- the meta_type of the method

          'title' -- the title of the method
        """

    def freezeCandidate(path, skins_tool=None):
        """ 'Freeze' a skin method / template into filesystem code.

        o 'path' is the path within the skins tool of the method /
          template to freeze.

        o 'skins_tool' is the tool to search;  if None, acquire it as
          'portal_skins'.  If a string, acquire it.

        o Return a mapping of values needed to generate a filesystem
          Python equivalent.  The mapping includes at least the key,
          'method_text', indented suitably for use as a method of a
          class declared at module scope (i.e. four spaces).  It may
          also include a key, 'module_scope_text', including text
          to be injected into the module at module scope (e.g., to
          have the template text available as a triple-quoted string).
        """

    def freezeCandidates(product_name, class_name, paths, skins_tool=None):
        """ Return a string for use as the __init__.py of the generated product.

        o 'product_name' is the name of the generated product.

        o 'class_name' is the name of the class whose instances will replace
          the skin methods.

        o 'paths' is the list of paths to the skin methods, relative to the
          skins tool.

        o 'skins_tool' is the tool to search;  if None, acquire it as
          'portal_skins'.  If a string, acquire it.
        """
