""" Frosbite freezer unit tests

$Id: test_freezer.py,v 1.5 2005/11/30 15:52:57 tseaver Exp $
"""
import unittest

class SkinFreezerTests(unittest.TestCase):

    def _getTargetClass(self):
        from Products.Frostbite.freezer import SkinFreezer
        return SkinFreezer

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_conforms_to_ISkinFreezer(self):
        from zope.interface.verify import verifyClass
        from Products.Frostbite.interfaces import ISkinFreezer
        verifyClass(ISkinFreezer, self._getTargetClass())

    def test_listCandidates_empty_tool(self):
        from Products.CMFCore.SkinsTool import SkinsTool

        tool = SkinsTool()

        freezer = self._makeOne()
        candidates = freezer.listCandidates(tool)
        self.assertEqual(len(candidates), 0)

    def test_listCandidates_tool_with_PythonScript(self):
        from Acquisition import aq_base
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PythonScripts.PythonScript import PythonScript

        tool = SkinsTool()
        tool._setObject('foo', PythonScript('foo'))
        script = tool._getOb('foo')
        script.write(_FOO_SCRIPT)

        freezer = self._makeOne()
        candidates = freezer.listCandidates(tool)
        self.assertEqual(len(candidates), 1)
        self.assertEqual(candidates[0]['id'], 'foo')
        self.assertEqual(candidates[0]['path'], 'foo')
        self.assertEqual(aq_base(candidates[0]['object']), aq_base(script))
        self.assertEqual(candidates[0]['product'], None)
        self.assertEqual(candidates[0]['meta_type'], PythonScript.meta_type)
        self.assertEqual(candidates[0]['title'], 'Foo Script')

    def test_listCandidates_tool_with_PageTemplate(self):
        from Acquisition import aq_base
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

        tool = SkinsTool()
        tool._setObject('bar', ZopePageTemplate('bar'))
        template = tool._getOb('bar')
        template.write(_BAR_TEMPLATE)
        template.title = 'Bar Template'

        freezer = self._makeOne()
        candidates = freezer.listCandidates(tool)
        self.assertEqual(len(candidates), 1)
        self.assertEqual(candidates[0]['id'], 'bar')
        self.assertEqual(candidates[0]['path'], 'bar')
        self.assertEqual(aq_base(candidates[0]['object']), aq_base(template))
        self.assertEqual(candidates[0]['product'], None)
        self.assertEqual(candidates[0]['meta_type'],
                         ZopePageTemplate.meta_type)
        self.assertEqual(candidates[0]['title'], 'Bar Template')

    def test_listCandidates_tool_with_FSPythonScript(self):
        from tempfile import NamedTemporaryFile
        from Acquisition import aq_base
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.CMFCore.FSPythonScript import FSPythonScript

        file = NamedTemporaryFile()
        file.write(_FOO_SCRIPT)
        file.flush()

        tool = SkinsTool()
        tool._setObject('foo', FSPythonScript('foo', filepath=file.name))
        script = tool._getOb('foo')

        freezer = self._makeOne()
        candidates = freezer.listCandidates(tool)
        self.assertEqual(len(candidates), 1)
        self.assertEqual(candidates[0]['id'], 'foo')
        self.assertEqual(candidates[0]['path'], 'foo')
        self.assertEqual(aq_base(candidates[0]['object']), aq_base(script))
        self.assertEqual(candidates[0]['product'], None)
        self.assertEqual(candidates[0]['meta_type'], FSPythonScript.meta_type)
        self.assertEqual(candidates[0]['title'], 'Foo Script')

    def test_listCandidates_tool_with_FSPageTemplate(self):
        from tempfile import NamedTemporaryFile
        from Acquisition import aq_base
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.CMFCore.FSPageTemplate import FSPageTemplate

        file = NamedTemporaryFile()
        file.write(_BAR_TEMPLATE)
        file.flush()

        tool = SkinsTool()
        tool._setObject('bar', FSPageTemplate('bar', filepath=file.name))
        template = tool._getOb('bar')
        template.title = 'Bar Template'

        freezer = self._makeOne()
        candidates = freezer.listCandidates(tool)
        self.assertEqual(len(candidates), 1)
        self.assertEqual(candidates[0]['id'], 'bar')
        self.assertEqual(candidates[0]['path'], 'bar')
        self.assertEqual(aq_base(candidates[0]['object']), aq_base(template))
        self.assertEqual(candidates[0]['product'], None)
        self.assertEqual(candidates[0]['meta_type'],
                         FSPageTemplate.meta_type)
        self.assertEqual(candidates[0]['title'], 'Bar Template')

    def test_listCandidates_tool_with_subfolder(self):
        from Acquisition import aq_base
        from OFS.Folder import Folder
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PythonScripts.PythonScript import PythonScript

        tool = SkinsTool()
        tool._setObject('qux', Folder('qux'))
        qux = tool._getOb('qux')
        qux._setObject('foo', PythonScript('foo'))
        script = qux._getOb('foo')
        script.write(_FOO_SCRIPT)

        tool.addSkinSelection('skinny', 'qux', make_default=True)

        freezer = self._makeOne()
        candidates = freezer.listCandidates(tool)
        self.assertEqual(len(candidates), 1)
        self.assertEqual(candidates[0]['id'], 'foo')
        self.assertEqual(candidates[0]['path'], 'qux/foo')
        self.assertEqual(aq_base(candidates[0]['object']), aq_base(script))
        self.assertEqual(candidates[0]['product'], None)
        self.assertEqual(candidates[0]['meta_type'], PythonScript.meta_type)
        self.assertEqual(candidates[0]['title'], 'Foo Script')

    def test_listCandidates_tool_with_skin_name(self):
        from Acquisition import aq_base
        from OFS.Folder import Folder
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PythonScripts.PythonScript import PythonScript

        tool = SkinsTool()
        tool._setObject('bam', Folder('bam'))
        bam = tool._getOb('bam')
        bam._setObject('foo', PythonScript('foo'))
        foo_bam = bam._getOb('foo')
        foo_bam.write(_FOO_SCRIPT.replace('Foo', 'FooBam'))
        bam._setObject('bambam', PythonScript('bambam'))

        tool._setObject('qux', Folder('qux'))
        qux = tool._getOb('qux')
        qux._setObject('foo', PythonScript('foo'))
        foo_qux = qux._getOb('foo')
        foo_qux.write(_FOO_SCRIPT.replace('Foo', 'FooQux'))
        qux._setObject('quxqux', PythonScript('quxqux'))

        tool.addSkinSelection('skinny', 'qux, bam', make_default=True)
        tool.addSkinSelection('fat', 'bam, qux')

        freezer = self._makeOne()

        candidates = freezer.listCandidates(tool, 'skinny')
        self.assertEqual(len(candidates), 3)
        paths = [x['path'] for x in candidates]
        self.failUnless('qux/foo' in paths)
        self.failUnless('qux/quxqux' in paths)
        self.failUnless('bam/bambam' in paths)

        candidates = freezer.listCandidates(tool, 'fat')
        self.assertEqual(len(candidates), 3)
        paths = [x['path'] for x in candidates]
        self.failUnless('bam/foo' in paths)
        self.failUnless('qux/quxqux' in paths)
        self.failUnless('bam/bambam' in paths)

    def test_listCandidates_acquire_tool_default_name(self):
        from Acquisition import aq_base
        from OFS.Folder import Folder
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PythonScripts.PythonScript import PythonScript

        site = Folder('site')
        tool = SkinsTool()
        tool._setObject('bam', Folder('bam'))
        bam = tool._getOb('bam')
        bam._setObject('foo', PythonScript('foo'))
        foo_bam = bam._getOb('foo')
        foo_bam.write(_FOO_SCRIPT.replace('Foo', 'FooBam'))
        bam._setObject('bambam', PythonScript('bambam'))

        tool._setObject('qux', Folder('qux'))
        qux = tool._getOb('qux')
        qux._setObject('foo', PythonScript('foo'))
        foo_qux = qux._getOb('foo')
        foo_qux.write(_FOO_SCRIPT.replace('Foo', 'FooQux'))
        qux._setObject('quxqux', PythonScript('quxqux'))

        tool.addSkinSelection('skinny', 'qux, bam', make_default=True)
        tool.addSkinSelection('fat', 'bam, qux')

        site._setObject('portal_skins', tool)

        freezer = self._makeOne().__of__(site)

        candidates = freezer.listCandidates()
        self.assertEqual(len(candidates), 3)
        paths = [x['path'] for x in candidates]
        self.failUnless('qux/foo' in paths)
        self.failUnless('qux/quxqux' in paths)
        self.failUnless('bam/bambam' in paths)

    def test_listCandidates_acquire_tool_explicit_name(self):
        from Acquisition import aq_base
        from OFS.Folder import Folder
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PythonScripts.PythonScript import PythonScript

        site = Folder('site')
        tool = SkinsTool()
        tool._setObject('bam', Folder('bam'))
        bam = tool._getOb('bam')
        bam._setObject('foo', PythonScript('foo'))
        foo_bam = bam._getOb('foo')
        foo_bam.write(_FOO_SCRIPT.replace('Foo', 'FooBam'))
        bam._setObject('bambam', PythonScript('bambam'))

        tool._setObject('qux', Folder('qux'))
        qux = tool._getOb('qux')
        qux._setObject('foo', PythonScript('foo'))
        foo_qux = qux._getOb('foo')
        foo_qux.write(_FOO_SCRIPT.replace('Foo', 'FooQux'))
        qux._setObject('quxqux', PythonScript('quxqux'))

        tool.addSkinSelection('skinny', 'qux, bam', make_default=True)
        tool.addSkinSelection('fat', 'bam, qux')

        site._setObject('portal_alternateskins', tool)

        freezer = self._makeOne().__of__(site)

        candidates = freezer.listCandidates('portal_alternateskins')
        self.assertEqual(len(candidates), 3)
        paths = [x['path'] for x in candidates]
        self.failUnless('qux/foo' in paths)
        self.failUnless('qux/quxqux' in paths)
        self.failUnless('bam/bambam' in paths)

    def test_freezeCandidate_tool_with_PythonScript_no_args(self):
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PythonScripts.PythonScript import PythonScript

        tool = SkinsTool()
        tool._setObject('foo', PythonScript('foo'))
        script = tool._getOb('foo')
        script.write(_FOO_SCRIPT)

        freezer = self._makeOne()
        frozen = freezer.freezeCandidate('foo', tool)
        self.assertEqual(frozen['method_text'], _FOO_SCRIPT_METHOD)
        self.failUnless(frozen.get('module_scope_text') is None)

    def test_freezeCandidate_tool_with_PythonScript_and_args(self):
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PythonScripts.PythonScript import PythonScript

        tool = SkinsTool()
        tool._setObject('qux', PythonScript('qux'))
        script = tool._getOb('qux')
        script.write(_QUX_SCRIPT)

        freezer = self._makeOne()
        frozen = freezer.freezeCandidate('qux', tool)
        self.assertEqual(frozen['method_text'], _QUX_SCRIPT_METHOD)
        self.failUnless(frozen.get('module_scope_text') is None)

    def test_freezeCandidate_tool_with_FSPythonScript(self):
        from tempfile import NamedTemporaryFile
        from Acquisition import aq_base
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.CMFCore.FSPythonScript import FSPythonScript

        file = NamedTemporaryFile()
        file.write(_FOO_SCRIPT)
        file.flush()

        tool = SkinsTool()
        tool._setObject('foo', FSPythonScript('foo', filepath=file.name))
        script = tool._getOb('foo')

        freezer = self._makeOne()
        frozen = freezer.freezeCandidate('foo', tool)
        self.assertEqual(frozen['method_text'], _FOO_SCRIPT_METHOD)
        self.failUnless(frozen.get('module_scope_text') is None)

    def test_freezeCandidate_tool_with_PageTemplate(self):
        from Acquisition import aq_base
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

        tool = SkinsTool()
        tool._setObject('bar', ZopePageTemplate('bar'))
        template = tool._getOb('bar')
        template.write(_BAR_TEMPLATE)
        template.title = 'Bar Template'

        freezer = self._makeOne()
        frozen = freezer.freezeCandidate('bar', tool)
        self.assertEqual(frozen['method_text'], _BAR_TEMPLATE_DECL)
        self.assertEqual(frozen['module_scope_text'],
                         _BAR_TEMPLATE_MODULE_SCOPE)

    def test_freezeCandidate_tool_with_FSPageTemplate(self):
        from tempfile import NamedTemporaryFile
        from Acquisition import aq_base
        from Products.CMFCore.SkinsTool import SkinsTool
        from Products.CMFCore.FSPageTemplate import FSPageTemplate

        file = NamedTemporaryFile()
        file.write(_BAR_TEMPLATE)
        file.flush()

        tool = SkinsTool()
        tool._setObject('bar', FSPageTemplate('bar', filepath=file.name))
        template = tool._getOb('bar')
        template.title = 'Bar Template'

        freezer = self._makeOne()
        frozen = freezer.freezeCandidate('bar', tool)
        self.assertEqual(frozen['method_text'], _BAR_TEMPLATE_DECL)
        self.assertEqual(frozen['module_scope_text'],
                         _BAR_TEMPLATE_MODULE_SCOPE)
 

_FOO_SCRIPT = """\
##Script (Python) "foo"
##title=Foo Script
##parameters=

return 'foo'
"""

_FOO_SCRIPT_METHOD = '''\
    security.declareProtected(View, 'foo')
    def foo(self):
        """ Foo Script
        """
        container = context = script = self
        return 'foo'
    foo = AcquireableMethod(foo)

'''

_BAR_TEMPLATE = """\
<html metal:use-macro="here/standard_template/macros/master">
<body>
<div metal:fill-slot="main">
MAIN SLOT CONTENT
</div>
</body>
</html>
"""

_BAR_TEMPLATE_DECL = """\
    security.declareProtected(View, 'bar')
    bar = ZopePageTemplate('bar', _BAR_TEMPLATE)
"""

_BAR_TEMPLATE_MODULE_SCOPE = '''\
_BAR_TEMPLATE = """\\
%s
"""
''' % _BAR_TEMPLATE

_QUX_SCRIPT = """\
##Script (Python) "qux"
##title=Qux Script
##parameters=baz, bam=None

return 'qux'
"""

_QUX_SCRIPT_METHOD = '''\
    security.declareProtected(View, 'qux')
    def qux(self, baz, bam=None):
        """ Qux Script
        """
        container = context = script = self
        return 'qux'
    qux = AcquireableMethod(qux)

'''


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SkinFreezerTests))
    return suite

if __name__ == '__main__':
    main(defaultTest='test_suite')
